#!/usr/bin/python3

import os
import json
import subprocess
import socket
import requests


while subprocess.call(["dpkg", "-s", "python3-openssl"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL) != 0:
    print("Installing pyOpenSSL...")
    res = subprocess.call(["apt-get", "install", "--yes", "python3-openssl"], stdout=subprocess.DEVNULL,
                          stderr=subprocess.DEVNULL)
    if res != 0:
        input(
            "Failed to install critical component (openssl)! Please ensure the machine is connected to the internet, then press enter to retry.")

from OpenSSL import crypto

licenseDir = "/var/IGP/license"
certFilename = "certificate.crt"
privKeyFilename = "privkey.pem"
cert = None
if not os.path.exists(licenseDir):
    os.mkdir(licenseDir)

store = crypto.X509Store()
rootcafile = open("/home/IGP/rootCA.crt")
store.add_cert(crypto.load_certificate(crypto.FILETYPE_PEM, rootcafile.read()))
rootcafile.close()

certfilelocation = os.path.join(licenseDir, certFilename)


def is_valid_cert() -> bool:
    if not os.path.exists(certfilelocation):
        return False

    f = open(certfilelocation)
    cert = crypto.load_certificate(crypto.FILETYPE_PEM, f.read())
    f.close()

    if cert is None:
        return False

    ctx = crypto.X509StoreContext(store, cert)
    try:
        ctx.verify_certificate()
        if cert.get_subject().commonName != socket.gethostname():
            print("Certificate is not issued for this machine!")
            return False
    except crypto.X509StoreContextError as e:
        print("Certificate is not valid: " + e)
        return False

    return True


with open("/var/lib/dbus/machine-id") as f:
    machineuid = f.readline()

while not is_valid_cert():
    activation_key = input("Enter Activation Key:\n")
    response = requests.get('https://rocket.imaxagaming.com/main/activate',
                            {"key": activation_key.upper(), "uid": machineuid})
    if response.status_code == requests.codes.ok or response.status_code == requests.codes.created:
        print("Got activation data - executing activation process...")
        if response.status_code == requests.codes.ok:
            print("\tRe-used activation key!")
        activation_data = response.json()
        print("\tMachine ID: " + activation_data["GAMING_MACHINE_ID"])
        print("\tMachine type: " + activation_data["MACHINE_TYPE"])
        print("\tLocation: " + activation_data["LOCATION"])
        print("\tJurisdiction: " + activation_data["JURISDICTION"])
        subprocess.call(['hostnamectl', 'set-hostname', activation_data["GAMING_MACHINE_ID"]])
        print("\tSaving activation information...", end="")
        with open("/var/IGP/machine-location", "w") as f:
            f.write(activation_data["LOCATION"])
        with open("/var/IGP/machine-type", "w") as f:
            f.write(activation_data["MACHINE_TYPE"])
        with open("/var/IGP/machine-jurisdiction", "w") as f:
            f.write(activation_data["JURISDICTION"])
        with open(certfilelocation, "w") as f:
            f.write(activation_data["CERT"])
        with open(os.path.join(licenseDir, privKeyFilename), "w") as f:
            f.write(activation_data["PRIVKEY"])
        print(" OK")
        print("\tCreating configuration files...", end="")
        nanoConfigFilepath = "/var/IGP/services_conf/yserver/providers/nano_localhost.conf"
        json.dump(dict({"name": activation_data["GAMING_MACHINE_ID"]}), open(nanoConfigFilepath, "w"))
        print(" OK")
        input("Activation successfull! Press enter to continue.\n")
    else:
        print("Activation failed - %d %s" % (response.status_code, response.reason))

exit(0)
